/*
 * Tiny Embedded JavaScript parser
 *
 * Copyright 2004, Broadcom Corporation
 * All Rights Reserved.
 * 
 * THIS SOFTWARE IS OFFERED "AS IS", AND BROADCOM GRANTS NO WARRANTIES OF ANY
 * KIND, EXPRESS OR IMPLIED, BY STATUTE, COMMUNICATION OR OTHERWISE. BROADCOM
 * SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A SPECIFIC PURPOSE OR NONINFRINGEMENT CONCERNING THIS SOFTWARE.
 *
 * $Id: ej.c,v 1.1 2007/04/22 05:13:25 lucas Exp $
 */
#if defined(SUPPORT_SHT)
#include <stdarg.h>
#include "boa.h"
#include "httpd.h"

static char * get_arg(char *args, char **next);
static void call(char *func, webs_t stream);

//#define DPRINTF		printf
#define DPRINTF		

/* Look for unquoted character within a string */
static char *
unqstrstr(char *haystack, char *needle)
{
	char *cur;
	int q;

	for (cur = haystack, q = 0;
	     cur < &haystack[strlen(haystack)] && !(!q && !strncmp(needle, cur, strlen(needle)));
	     cur++) {
		if (*cur == '"')
			q ? q-- : q++;
	}
	return (cur < &haystack[strlen(haystack)]) ? cur : NULL;
}

static char *
get_arg(char *args, char **next)
{
	char *arg, *end;

	/* Parse out arg, ... */
	if (!(end = unqstrstr(args, ","))) {
		end = args + strlen(args);
		*next = NULL;
	} else
		*next = end + 1;

	/* Skip whitespace and quotation marks on either end of arg */
	for (arg = args; isspace((int)*arg) || *arg == '"'; arg++);
	for (*end-- = '\0'; isspace((int)*end) || *end == '"'; end--)
		*end = '\0';

	return arg;
}


static void
call(char *func, webs_t	stream)
{
	char *args, *end, *next;
	int argc;
	char * argv[16];
	struct ej_handler *handler;

	/* Parse out ( args ) */
	if (!(args = strchr(func, '(')))
		return;
	if (!(end = unqstrstr(func, ")")))
		return;
	*args++ = *end = '\0';

	/* Set up argv list */
	for (argc = 0; argc < 16 && args && *args; argc++, args = next) {
		if (!(argv[argc] = get_arg(args, &next)))
			break;
	}

	/* Call handler */
	for (handler = &ej_handlers[0]; handler->pattern; handler++) {
		if (strncmp(handler->pattern, func, strlen(handler->pattern)) == 0)
			handler->output(0, stream, argc, argv);
	}
}

int
ejArgs(int argc, char **argv, char *fmt, ...)
{
	va_list	ap;
	int arg;
	char *c;

	if (!argv)
		return 0;

	va_start(ap, fmt);
	for (arg = 0, c = fmt; c && *c && arg < argc;) {
		if (*c++ != '%')
			continue;
		switch (*c) {
		case 'd':
			*(va_arg(ap, int *)) = atoi(argv[arg]);
			break;
		case 's':
			*(va_arg(ap, char **)) = argv[arg];
			break;
		}
		arg++;
	}
	va_end(ap);

	return arg;
}

int handle_sht(webs_t req, int size, int need_filepos)
{
    	int bytes_readed = -1;
        char c, *func = NULL, *end = NULL;
        int pos=0;
        static char *asp=NULL;
	
	if (need_filepos)
		pos = req->filepos;
		
	while (++bytes_readed < size) 
	{
		c = *(req->data_mem + pos + bytes_readed);	
		
		if (req->remained_len == (sizeof(req->pattern) - 1))
			req->remained_len = 0;
			
		/* Add to pattern space */
		req->pattern[req->remained_len++] = c;
		req->pattern[req->remained_len] = '\0';
		if (req->remained_len == (sizeof(req->pattern) - 1))
			goto release;


		/* Look for <% ... */
		if (!asp && !strncmp(req->pattern, "<%", req->remained_len)) 
		{
			if (req->remained_len == 2)
				asp = req->pattern + 2;
			continue;
		}

		/* Look for ... %> */
		if (asp) 
		{
			if (unqstrstr(asp, "%>")) 
			{
				DPRINTF("found <\%%tag\%%> %s\n", req->pattern);
				for (func = asp; func < &(req->pattern[req->remained_len]); func = end) 
				{
					/* Skip initial whitespace */
					for (; isspace((int)*func); func++);
					if (!(end = unqstrstr(func, ";")))
						break;
					*end++ = '\0';

					/* Call function */
					call(func, req);
				}	
				//req_write(req, req->pattern);
				asp = NULL;
				req->remained_len = 0;
			}
			continue;
		}

release:
		/* Release pattern space */
		req_write(req, req->pattern);
		asp = NULL;		
		req->remained_len = 0;
	}  
        return size;     
}
#endif
