/*
 *  Boa, an http server
 *  Copyright (C) 1995 Paul Phillips <psp@well.com>
 *  Some changes Copyright (C) 1996,97 Larry Doolittle <ldoolitt@jlab.org>
 *  Some changes Copyright (C) 1996-99 Jon Nelson <jnelson@boa.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 1, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

/* $Id: request.c,v 1.100 2001/07/18 03:40:10 jnelson Exp $*/

#include "boa.h"
#include <netinet/tcp.h>
#include <arpa/inet.h>          /* inet_ntoa */
#include <stddef.h>

#include <sys/ioctl.h>      /* ioctl */
#include <net/if.h>     /* struct ifreq */

#define WANLAN_CONFLICT_PAGE "/cgi-bin/LIPChange.html"
static char auth_userpass[0x80];
static int file_set(const char *name, const char *value);

int sockbufsize = SOCKETBUF_SIZE;
int total_connections;

/*
 * Name: new_request
 e Description: Obtains a request struct off the free list, or if the
 * free list is empty, allocates memory
 *
 * Return value: pointer to initialized request
 */

request *new_request(void)
{
    request *req;

    if (request_free) {
        req = request_free;     /* first on free list */
        dequeue(&request_free, request_free); /* dequeue the head */
    } else {
        req = (request *) malloc(sizeof (request));
        if (!req) {
            log_error_mesg(__FILE__, __LINE__,
                           "out of memory allocating for new request structure");
            exit(errno);
        }
    }

    memset(req, 0, offsetof(request, buffer) + 1);

    return req;
}

#if defined(DNI_AUTH)
char *
getMacAddr(char *ipaddr_)
{
    FILE *pF;
    char line[120], temp[30], *pStrStart, *pStrEnd;
    static char arp_mac[MAX_IP_LEN];
    unsigned long int addr;


    memset(arp_mac,0, MAX_IP_LEN);
    if(ipaddr_ != NULL)
        addr=inet_addr(ipaddr_);
   else
       printf("bad ipaddress ....\n");

    pF=fopen("/proc/net/arp", "r");
    if(pF != NULL){
        fgets(line,120,pF);//skip describe
        while(fgets(line,120,pF))
        {
            memset(temp, 0, 30);
            pStrEnd=strchr(line,0x20);
            memcpy(temp,line,pStrEnd-line);
            if(addr == inet_addr(temp)){//find same ip
                pStrStart=strchr(pStrEnd+1,':')-2;
                pStrEnd=rindex(pStrStart+1,':')+3;
                memcpy(arp_mac,pStrStart,pStrEnd-pStrStart);
            }
        }
        fclose(pF);
    }else{
        printf("not open arp table..\n");
    }
    return arp_mac;
}
#endif

/*
 * Name: get_request
 *
 * Description: Polls the server socket for a request.  If one exists,
 * does some basic initialization and adds it to the ready queue;.
 */

void get_request(int server_s)
{
    int fd;                     /* socket */
    struct SOCKADDR remote_addr; /* address */
    int remote_addrlen = sizeof (struct sockaddr_in);
    request *conn;              /* connection */
    int len;
    static int system_bufsize = 0; /* Default size of SNDBUF given by system */

    remote_addr.S_FAMILY = 0xdead;
    fd = accept(server_s, (struct sockaddr *) &remote_addr,
                &remote_addrlen);

    if (fd == -1) {
        if (errno != EAGAIN && errno != EWOULDBLOCK) /* no requests */
            log_error_mesg(__FILE__, __LINE__, "accept");
        else
            pending_requests = 0;
        return;
    }
    if (fd >= FD_SETSIZE) {
        log_error_mesg(__FILE__, __LINE__, "Got fd >= FD_SETSIZE.");
        close(fd);
    return;
    }
#ifdef DEBUGNONINET
    /* This shows up due to race conditions in some Linux kernels
       when the client closes the socket sometime between
       the select() and accept() syscalls.
       Code and description by Larry Doolittle <ldoolitt@boa.org>
     */
#define HEX(x) (((x)>9)?(('a'-10)+(x)):('0'+(x)))
    if (remote_addr.sin_family != AF_INET) {
        struct sockaddr *bogus = (struct sockaddr *) &remote_addr;
        char *ap, ablock[44];
        int i;
        close(fd);
        log_error_time();
        for (ap = ablock, i = 0; i < remote_addrlen && i < 14; i++) {
            *ap++ = ' ';
            *ap++ = HEX((bogus->sa_data[i] >> 4) & 0x0f);
            *ap++ = HEX(bogus->sa_data[i] & 0x0f);
        }
        *ap = '\0';
        fprintf(stderr, "non-INET connection attempt: socket %d, "
                "sa_family = %hu, sa_data[%d] = %s\n",
                fd, bogus->sa_family, remote_addrlen, ablock);
        return;
    }
#endif

/* XXX Either delete this, or document why it's needed */
/* Pointed out 3-Oct-1999 by Paul Saab <paul@mu.org> */
#ifdef REUSE_EACH_CLIENT_CONNECTION_SOCKET
    if ((setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, (void *) &sock_opt,
                    sizeof (sock_opt))) == -1) {
        log_error_mesg(__FILE__, __LINE__,
                       "setsockopt: unable to set SO_REUSEADDR");
        exit(errno);
    }
#endif

    conn = new_request();
    conn->fd = fd;
    conn->status = READ_HEADER;
    conn->header_line = conn->client_stream;
    conn->time_last = current_time;
    conn->kacount = ka_max;

    /* fill in local_ip_addr if relevant */
    if (virtualhost) {
        struct SOCKADDR salocal;
        int dummy = sizeof (salocal);
        if (getsockname(conn->fd, (struct sockaddr *) &salocal, &dummy) ==
            -1) {
            log_error_mesg(__FILE__, __LINE__, "getsockname");
            exit(errno);
        }
        ascii_sockaddr(&salocal, conn->local_ip_addr, NI_MAXHOST);
    }

    /* nonblocking socket */
    if (fcntl(conn->fd, F_SETFL, NOBLOCK) == -1)
        log_error_mesg(__FILE__, __LINE__,
                       "fcntl: unable to set new socket to non-block");

    /* set close on exec to true */
    if (fcntl(conn->fd, F_SETFD, 1) == -1)
        log_error_mesg(__FILE__, __LINE__,
                       "fctnl: unable to set close-on-exec for new socket");

    /* Increase buffer size if we have to.
     * Only ask the system the buffer size on the first request,
     * and assume all subsequent sockets have the same size.
     */
    if (system_bufsize == 0) {
        len = sizeof (system_bufsize);
        if (getsockopt
            (conn->fd, SOL_SOCKET, SO_SNDBUF, &system_bufsize, &len) == 0
            && len == sizeof (system_bufsize)) {
            /*
               fprintf(stderr, "%sgetsockopt reports SNDBUF %d\n",
               get_commonlog_time(), system_bufsize);
             */
            ;
        } else {
            log_error_mesg(__FILE__, __LINE__, "getsockopt(SNDBUF)");
            system_bufsize = 1;
        }
    }
    if (system_bufsize < sockbufsize) {
        if (setsockopt
            (conn->fd, SOL_SOCKET, SO_SNDBUF, (void *) &sockbufsize,
             sizeof (sockbufsize)) == -1) {
            log_error_mesg(__FILE__, __LINE__,
                           "setsockopt: unable to set socket buffer size");
#ifdef DIE_ON_ERROR_TUNING_SNDBUF
            exit(errno);
#endif
        }
    }

    /* for log file and possible use by CGI programs */
    ascii_sockaddr(&remote_addr, conn->remote_ip_addr, NI_MAXHOST);

    /* for possible use by CGI programs */
    conn->remote_port = ntohs(remote_addr.sin_port);

    status.requests++;

#ifdef USE_TCPNODELAY
    /* Thanks to Jef Poskanzer <jef@acme.com> for this tweak */
    {
        int one = 1;
        if (setsockopt(conn->fd, IPPROTO_TCP, TCP_NODELAY,
                       (void *) &one, sizeof (one)) == -1) {
            log_error_mesg(__FILE__, __LINE__,
                           "setsockopt: unable to set TCP_NODELAY");
            exit(errno);
        }

    }
#endif

    if (conn->fd > max_connections) {
        send_r_service_unavailable(conn);
        conn->status = DONE;
        pending_requests = 0;
    }

    total_connections++;
    enqueue(&request_ready, conn);
}


/*
 * Name: free_request
 *
 * Description: Deallocates memory for a finished request and closes
 * down socket.
 */

static void free_request(request ** list_head_addr, request * req)
{
    /* free_request should *never* get called by anything but
       process_requests */

    if (req->buffer_end && req->status != DEAD) {
        req->status = DONE;
        return;
    }
    /* put request on the free list */
    dequeue(list_head_addr, req); /* dequeue from ready or block list */

    FD_CLR(req->fd, &block_read_fdset);
    FD_CLR(req->fd, &block_write_fdset);
    FD_CLR(req->data_fd, &block_read_fdset);
    FD_CLR(req->post_data_fd, &block_write_fdset);

    if (req->logline)           /* access log */
        log_access(req);

    if (req->mmap_entry_var)
        release_mmap(req->mmap_entry_var);
    else if (req->data_mem)
        munmap(req->data_mem, req->filesize);

    if (req->data_fd)
        close(req->data_fd);

    if (req->post_data_fd)
        close(req->post_data_fd);

    if (req->response_status >= 400)
        status.errors++;

    if (req->cgi_env) {
        int i;
        for (i = COMMON_CGI_COUNT; i < req->cgi_env_index; ++i)
            if (req->cgi_env[i])
                free(req->cgi_env[i]);
        free(req->cgi_env);
    }
    if (req->pathname)
        free(req->pathname);
    if (req->path_info)
        free(req->path_info);
    if (req->path_translated)
        free(req->path_translated);
    if (req->script_name)
        free(req->script_name);

    if ((req->keepalive == KA_ACTIVE) &&
        (req->response_status < 500) && req->kacount > 0) {

        request *conn = new_request();
        conn->fd = req->fd;
        conn->status = READ_HEADER;
        conn->header_line = conn->client_stream;
        conn->kacount = req->kacount - 1;

        /* close enough and we avoid a call to time(NULL) */
        conn->time_last = req->time_last;

        /* for log file and possible use by CGI programs */
        memcpy(conn->remote_ip_addr, req->remote_ip_addr, NI_MAXHOST);
        memcpy(conn->local_ip_addr, req->local_ip_addr, NI_MAXHOST);

        /* for possible use by CGI programs */
        conn->remote_port = req->remote_port;

        status.requests++;

        {
            int bytes_to_move;

            /* we haven't parsed beyond req->parse_pos, so... */
            bytes_to_move = req->client_stream_pos - req->parse_pos;

            if (bytes_to_move) {
                memcpy(conn->client_stream,
                       req->client_stream + req->parse_pos, bytes_to_move);
                conn->client_stream_pos = bytes_to_move;
                FD_CLR(conn->fd, &block_read_fdset);
                enqueue(&request_ready, conn);
            } else {
                FD_SET(conn->fd, &block_read_fdset);
                enqueue(&request_block, conn);
            }
        }
    } else {
        /*
         While debugging some weird errors, Jon Nelson learned that
         some versions of Netscape Navigator break the
         HTTP specification.

         Some research on the issue brought up:

         http://www.apache.org/docs/misc/known_client_problems.html

         As quoted here:

         "
         Trailing CRLF on POSTs

         This is a legacy issue. The CERN webserver required POST
         data to have an extra CRLF following it. Thus many
         clients send an extra CRLF that is not included in the
         Content-Length of the request. Apache works around this
         problem by eating any empty lines which appear before a
         request.
         "

         Boa will (for now) hack around this stupid bug in Netscape
         (and Internet Exploder)
         by reading up to 32k after the connection is all but closed.
         This should eliminate any remaining spurious crlf sent
         by the client.

         Building bugs *into* software to be compatable is
         just plain wrong
         */

        if (req->method == M_POST || req->method == M_PUT) {
            char buf[32768];
            read(req->fd, buf, 32768);
        }
        close(req->fd);
        total_connections--;
    }

    enqueue(&request_free, req);

    return;
}

static int file_set(const char *name, const char *value)
{
    nvram_set(name, value);
    return 0;
}

/*
 * Name: process_requests
 *
 * Description: Iterates through the ready queue, passing each request
 * to the appropriate handler for processing.  It monitors the
 * return value from handler functions, all of which return -1
 * to indicate a block, 0 on completion and 1 to remain on the
 * ready list for more procesing.
 */

void process_requests(int server_s)
{
    //char remote_ip[NI_MAXHOST];
    int retval = 0;
    request *current, *trailer;

    if (pending_requests) {
        get_request(server_s);
#ifdef ORIGINAL_BEHAVIOR
        pending_requests = 0;
#endif
    }
    current = request_ready;
 /*   if( current && memcmp(remote_ip, current->remote_ip_addr, NI_MAXHOST) )
    {
        memcpy(remote_ip, current->remote_ip_addr, NI_MAXHOST);
        file_set("remote_ip", remote_ip);
        system("mac=$(/bin/cat /proc/net/arp | /bin/sed -n \"/^$(/usr/sbin/nvram get remote_ip)/p\" | /usr/bin/awk '{print $4}'); /usr/sbin/nvram set wan_remote_mac=$mac");
    } */


    while (current) {
        time(&current_time);
        if (current->buffer_end && /* there is data in the buffer */
            current->status != DEAD && current->status != DONE) {
            retval = req_flush(current);
            /*
             * retval can be -2=error, -1=blocked, or bytes left
             */
            if (retval == -2) { /* error */
                current->status = DEAD;
                retval = 0;
            } else if (retval >= 0) {
                /* notice the >= which is different from below?
                   Here, we may just be flushing headers.
                   We don't want to return 0 because we are not DONE
                   or DEAD */

                retval = 1;
            }
        } else {
            switch (current->status) {
            case READ_HEADER:
            case ONE_CR:
            case ONE_LF:
            case TWO_CR:
                retval = read_header(current);
                break;
            case BODY_READ:
                retval = read_body(current);
                break;
            case BODY_WRITE:
                retval = write_body(current);
                break;
            case WRITE:
                retval = process_get(current);
                break;
            case PIPE_READ:
                retval = read_from_pipe(current);
                break;
            case PIPE_WRITE:
                retval = write_from_pipe(current);
                break;
            case DONE:
                /* a non-status that will terminate the request */
                retval = req_flush(current);
                /*
                 * retval can be -2=error, -1=blocked, or bytes left
                 */
                if (retval == -2) { /* error */
                    current->status = DEAD;
                    retval = 0;
                } else if (retval > 0) {
                    retval = 1;
                }
                break;
            case DEAD:
                retval = 0;
                current->buffer_end = 0;
                SQUASH_KA(current);
                break;
            default:
                retval = 0;
                fprintf(stderr, "Unknown status (%d), "
                        "closing!\n", current->status);
                current->status = DEAD;
                break;
            }

        }

        if (lame_duck_mode)
            SQUASH_KA(current);

        /* we put this here instead of after the switch so that
         * if we are on the last request, and get_request is successful,
         * current->next is valid!
         */
        if (pending_requests)
            get_request(server_s);

        switch (retval) {
        case -1:               /* request blocked */
            trailer = current;
            current = current->next;
            block_request(trailer);
            break;
        case 0:                /* request complete */
            current->time_last = current_time;
            trailer = current;
            current = current->next;
            free_request(&request_ready, trailer);
            break;
        case 1:                /* more to do */
            current->time_last = current_time;
            current = current->next;
            break;
        default:
            log_error_time();
            fprintf(stderr, "Unknown retval in process.c - "
                    "Status: %d, retval: %d\n", current->status, retval);
            current = current->next;
            break;
        }
    }
}

/*
 * Name: process_logline
 *
 * Description: This is called with the first req->header_line received
 * by a request, called "logline" because it is logged to a file.
 * It is parsed to determine request type and method, then passed to
 * translate_uri for further parsing.  Also sets up CGI environment if
 * needed.
 */

int process_logline(request * req)
{
    char *stop, *stop2;
    static char *SIMPLE_HTTP_VERSION = "HTTP/0.9";

    req->logline = req->client_stream;
    if (!memcmp(req->logline, "GET ", 4))
        req->method = M_GET;
    else if (!memcmp(req->logline, "HEAD ", 5))
        /* head is just get w/no body */
        req->method = M_HEAD;
    else if (!memcmp(req->logline, "POST ", 5))
        req->method = M_POST;
    else {
        log_error_time();
        fprintf(stderr, "malformed request: \"%s\"\n", req->logline);
        send_r_bad_request(req);
        return 0;
    }

    req->http_version = SIMPLE_HTTP_VERSION;
    req->simple = 1;

    /* Guaranteed to find ' ' since we matched a method above */
    stop = req->logline + 3;
    if (*stop != ' ')
        ++stop;

    /* scan to start of non-whitespace */
    while (*(++stop) == ' ');

    stop2 = stop;

    /* scan to end of non-whitespace */
    while (*stop2 != '\0' && *stop2 != ' ')
        ++stop2;

    if (stop2 - stop > MAX_HEADER_LENGTH) {
        log_error_time();
        fprintf(stderr, "URI too long %d: \"%s\"\n", MAX_HEADER_LENGTH,
                req->logline);
        send_r_bad_request(req);
        return 0;
    }
    memcpy(req->request_uri, stop, stop2 - stop);
    req->request_uri[stop2 - stop] = '\0';
#ifdef WAN_LAN_CONFLICT
    if(open_hijack_conflict(WANLAN_CONFLICT)) {
		memcpy(req->request_uri, WANLAN_CONFLICT_PAGE, sizeof(WANLAN_CONFLICT_PAGE) );
        system("/etc/rc.d/dns_hijack.sh stop");
        nvram_unset(WANLAN_CONFLICT);
        system("/etc/rc.d/dns_hijack.sh start");
        system("/etc/rc.d/conflict.sh stop");
	}
#endif

    if (*stop2 == ' ') {
        /* if found, we should get an HTTP/x.x */
        unsigned int p1, p2;

        if (sscanf(++stop2, "HTTP/%u.%u", &p1, &p2) == 2) {
            /* HTTP/{0.9,1.0,1.1} */
            if (p1 == 1 && (p2 == 0 || p2 == 1)) {
                req->http_version = stop2;
                req->simple = 0;
            } else if (p1 > 1 || (p1 != 0 && p2 > 1)) {
                goto BAD_VERSION;
            }
        } else {
            goto BAD_VERSION;
        }
    }

    if (req->method == M_HEAD && req->simple) {
        send_r_bad_request(req);
        return 0;
    }
    if (translate_uri(req) == 0) { /* unescape, parse uri */
        SQUASH_KA(req);
        return 0;               /* failure, close down */
    }
    if (req->is_cgi)
        create_env(req);
    return 1;

BAD_VERSION:
    log_error_time();
    fprintf(stderr, "bogus HTTP version: \"%s\"\n", stop2);
    send_r_bad_request(req);
    return 0;
}

/*
 * Name: process_header_end
 *
 * Description: takes a request and performs some final checking before
 * init_cgi or init_get
 * Returns 0 for error or NPH, or 1 for success
 */

int base64decode(void *dst,char *src,int maxlen)
{
    int bitval,bits;
    int val;
    int len,x,y;

    len = strlen(src);
    bitval=0;
    bits=0;
    y=0;

    for(x=0;x<len;x++)  {
        if ((src[x]>='A')&&(src[x]<='Z')) val=src[x]-'A'; else
            if ((src[x]>='a')&&(src[x]<='z')) val=src[x]-'a'+26; else
                if ((src[x]>='0')&&(src[x]<='9')) val=src[x]-'0'+52; else
                    if (src[x]=='+') val=62; else
                        if (src[x]=='-') val=63; else
                            val=-1;
        if (val>=0)         {
            bitval=bitval<<6;
            bitval+=val;
            bits+=6;
            while (bits>=8)             {
                if (y<maxlen)
                    ((char *)dst)[y++]=(bitval>>(bits-8))&0xFF;
                bits-=8;
                bitval &= (1<<bits)-1;
            }
        }
    }
    if (y<maxlen)
        ((char *)dst)[y++]=0;
    return y;
}

#define FROM_LAN 0
#define FROM_WAN 1
#define LAN_INTERFACE "br0"
//#define LAN_INTERFACE "br1"
char* from_subnet(request *req, char *login_ipaddr)
{
    int fd;
    static char *from[] = { "admin", "remote" };
    uint32_t addr, mask, remote_ip;
    struct sockaddr_in *dev_ip;
    struct ifreq ifr;

    if ((fd = socket(AF_INET, SOCK_RAW, IPPROTO_RAW)) >= 0){
        ifr.ifr_addr.sa_family = AF_INET;
        //strcpy(ifr.ifr_name, LAN_INTERFACE);
        strcpy(ifr.ifr_name, (nvram_get("lan_ifname") ? nvram_get("lan_ifname") : LAN_INTERFACE));

        if (ioctl(fd, SIOCGIFADDR, &ifr) == 0){
            dev_ip = (struct sockaddr_in *) &ifr.ifr_addr;
            addr = dev_ip->sin_addr.s_addr;
        } else {
            close(fd); return from[FROM_LAN];
        }

        if (ioctl(fd, SIOCGIFNETMASK, &ifr) == 0){
            dev_ip = (struct sockaddr_in *) &ifr.ifr_addr;
            mask = dev_ip->sin_addr.s_addr;
        } else {
            close(fd); return from[FROM_LAN];
        }

        if (req != NULL)
            inet_aton( req->remote_ip_addr, (struct in_addr *)&remote_ip);
        else
            inet_aton( login_ipaddr, (struct in_addr *)&remote_ip);
        if ( (addr & mask) == (remote_ip & mask) ){
            close(fd); return from[FROM_LAN];
        } else {
            close(fd); return from[FROM_WAN];
        }
        close(fd);
    }
}

#define MGMT_LOGIN_LEVEL	1

#if defined(DNI_AUTH)

#define ADMIN_LEVEL 	2
#define USER_LEVEL	1
#define GUEST_LEVEL	0

int g_bLogin = 0;

client_req_t client_req = {0};

void auth_preload(void)
{

	char strbuf[10];
    	//If an authorized host reboots the machine,
        //it is recorded and doesn't need another authoriaztion.
        if ((nvram_get("auth_reboot")!=NULL) && strcmp(nvram_get("auth_reboot"), "1") == 0) {
            char *peerip, *peerMac;

            peerip = nvram_get("auth_ip");
            peerMac = nvram_safe_get("auth_mac");
            if (peerip != NULL) {
                    memset(&client_req.client_ip, 0x0, MAX_IP_LEN);
                    strncpy(client_req.client_ip, peerip, MAX_IP_LEN);
                    client_req.authed_send=1;
                    client_req.lastaccess_time = GetSysUpTime();
                    client_req.mgmt_level = atoi(nvram_safe_get("mgmt_level"));
                    g_bLogin =1;
                    memset(&client_req.client_macaddress, 0x0, MAX_IP_LEN);
                    strncpy(&client_req.client_macaddress, peerMac, MAX_IP_LEN);
                }
            //nvram_set("auth_ip", "");
            nvram_set("auth_reboot", "0");
            nvram_commit();
            nvram_set("user_login","1");
	    	
        syslog(6, "[%s login] from source %s,", from_subnet(NULL, peerip), peerip);

        }
#if defined(MGMT_LOGIN_LEVEL)
	sprintf(strbuf, "%d", client_req.mgmt_level);
	nvram_set("mgmt_level", strbuf);
#else
	nvram_set("mgmt_level", "2");
#endif
	
}

int clr_http_client(void)
{
    g_bLogin = 0;
    memset((char *)&client_req, 0x0, sizeof(client_req_t));
}

int copy_httpc_info_to_client(char *ptr)
{
    memcpy((char *)&client_req, ptr, sizeof(client_req_t));
}


#define HTTP_LOGIN_TIMEOUT  (5 * 60)    /* 5 minutes */

int httpd_check_auth(char *peerIP)
{
    char peerMacaddress[MAX_IP_LEN];

    if(   g_bLogin &&
        ((GetSysUpTime() - client_req.lastaccess_time) > HTTP_LOGIN_TIMEOUT) ) {
                g_bLogin = 0;
                memset((char *)&client_req, 0x0, sizeof(client_req));
    }

    if( g_bLogin )
    {
        /* To do check mac address */
        /*
        system("mac=$(/bin/cat /proc/net/arp | /bin/sed -n \"/^$(/usr/sbin/nvram get remote_ip_now)/p\" | /usr/bin/awk '{print $4}'); /usr/sbin/nvram set wan_remote_mac_now=$mac");
        strcpy(remote_mac, nvram_safe_get("wan_remote_mac"));
        strcpy(remote_mac_now, nvram_safe_get("wan_remote_mac_now"));
        */
        strncpy(peerMacaddress,getMacAddr(peerIP), MAX_IP_LEN);
        if (strcmp(client_req.client_ip, peerIP) == 0)
        {
            client_req.lastaccess_time=GetSysUpTime();
            return 1;
        }else if( strcmp(client_req.client_macaddress, peerMacaddress) == 0){
            memset(&client_req.client_ip, 0x0, MAX_IP_LEN);
            strncpy(client_req.client_ip, peerIP, MAX_IP_LEN);
            nvram_set("auth_ip", client_req.client_ip);
            client_req.lastaccess_time=GetSysUpTime();
            return 1;
        }else
            return 0;
    }
    else
        return 0;
}

/* return mgmt_level */
int auth_check_userpass(char *user,char *pass,request * req)
{
	if(user == NULL)
		return -1;
	if(pass == NULL)
	{
		if((strcmp(nvram_get("user1"),user)==0)&&(nvram_get("password1")==NULL)) 
		{
			
			return ADMIN_LEVEL;
		}
		else if((strcmp(nvram_get("user2"),user)==0)&& (nvram_get("password2")==NULL)) 
		{
			return USER_LEVEL;
		}
		else if((strcmp(nvram_get("user3"),user)==0)&&(nvram_get("password3")==NULL)) 
		{
			return GUEST_LEVEL;
		}	
	}
	else
	{
		if((nvram_get("user1")!=NULL)&&(strcmp(nvram_get("user1"),user)==0)&&(strcmp(nvram_get("password1"),pass)==0)) 
		{
			return ADMIN_LEVEL;
		}
		else if((nvram_get("user2")!=NULL)&&(strcmp(nvram_get("user2"),user)==0)&&(strcmp(nvram_get("password2"),pass)==0)) 
		{
			return USER_LEVEL;
		}
		else if((nvram_get("user3")!=NULL)&&(strcmp(nvram_get("user3"),user)==0)&&(strcmp(nvram_get("password3"),pass)==0)) 
		{
			return GUEST_LEVEL;
		}
	}
	
	if (((nvram_get("user1")!=NULL)&&(strcmp(nvram_get("user1"),user)==0)) || 
		((nvram_get("user2")!=NULL)&&(strcmp(nvram_get("user2"),user)==0)) || 
		((nvram_get("user3")!=NULL)&&(strcmp(nvram_get("user3"),user)==0)))
	{
		return -1;
	}
	else return -2;
}

int auth_authorize(request *req)
{
    char password[40], user[40];
        char admin_passwd[80]="";
    char remote_mac[18], remote_mac_now[18];

    int do_auth = 0;

        char auth_userpass[0x80];
        int i = 0;

        char non_block[][50]={
        "/duplicate.html",
        "/logout.html",
        "/string.js",
        "/debuginfo.htm",
        "violation_page.html",
        "ErrorAuthCancel.html",
        "currentsetting.htm",
        "showMenu.js",
        "styles.css",
        "LIPChange.html",
	".gif",
	".jpg",
	".js",
	".css",
	".ico",
        NULL};
        
        char easy_setup[][50]={
        "/easy_wizard.html",
        "/easy_wizard_detwan.html",
        "/easy_wizard_finish.html",
        "/easy_wizard_nowan.html",
        "/ew_bas_ether.html",
        "/ew_bas_inter.html",
        "/ew_bas_pppoe.html",
        "/ew_bas_pptp.html",
        "/ew_bas_top.html",
        "/ew_bas_update.html",
        "/ew_detwan_wait.html",
        "/ew_connection_info.html",
        "/ew_wlan.html",
        "/bas_buttom.html",
        "ew_top.html",
        "noauth_no_commit.cgi",
        "noauth_setobject.cgi",
		"ut_upgrade.cgi",
        NULL};
				
        for(i =0; *non_block[i] != NULL; i++)
        {
            if(strstr(req->request_uri, non_block[i])){
		if (strstr(req->request_uri, "logout.html")) {
        		clr_http_client();
        		nvram_unset("user_login");
        		nvram_unset("err_auth");
		}
                return 1;
            }
        }
      
			for(i=0; *easy_setup[i]!=NULL; i++)
			{	
				if(strstr(req->request_uri, easy_setup[i]))
					return 1;
			}

    if((nvram_get("user_login")!=NULL) && strcmp(nvram_get("user_login"), "0")==0)
    {
        clr_http_client();
        nvram_unset("user_login");
    }

    if((nvram_get("err_auth")!=NULL) && strcmp(nvram_get("err_auth"), "1")==0)
    {
        clr_http_client();
        //client_req.authed_send=0;
        nvram_unset("err_auth");
    }

    if (g_bLogin == 1) {
        if (httpd_check_auth(req->remote_ip_addr) == 0) {
            if (g_bLogin == 0) {
            	client_req.authed_send = 1;
	            strncpy(client_req.client_ip, req->remote_ip_addr, sizeof(client_req.client_ip));
		    if (strcmp(nvram_safe_get("password1"), "") != 0) {	
	        	    send_r_unauthorized(req,server_name);
		            return 0;
		    }	
            }
            else {
#if 0
            char path[64] = "/usr/local/www/serverbusy.html";

            send_r_request_ok(req);
            do_ej(path, req);
            return 0;
#endif
            	send_r_unauthorizedlogin(req, client_req.client_ip);
            /*syslog(6, "[%s login failure] from source %s,", from_subnet(req, NULL), req->remote_ip_addr);*/
                return 0;
            }
        }
    }

    if ((client_req.authed_send == 0) || (client_req.client_ip[0] == '\0')) {
        memset(client_req.client_ip, 0x0, MAX_IP_LEN);
        strncpy(client_req.client_ip, req->remote_ip_addr, sizeof(client_req.client_ip));
        client_req.authed_send = 1;

	if (strcmp(nvram_safe_get("password1"), "") != 0) {	
	        send_r_unauthorized(req,server_name);
        	return 0;
	}
    }

    if ((req->authorization == NULL) || strcmp(client_req.client_ip, req->remote_ip_addr) ) {
        memset(client_req.client_ip, 0x0, MAX_IP_LEN);
        strncpy(client_req.client_ip, req->remote_ip_addr, sizeof(client_req.client_ip));
        client_req.authed_send = 1;

        if (strcmp(nvram_safe_get("password1"), "") != 0) {	
	        send_r_unauthorized(req,server_name);
        	return 0;
	}
    }

    if (strcmp(nvram_safe_get("password1"), "") == 0) {
 	client_req.authed_send = 1;
	goto auth_sucess;
    }	

    if (strncasecmp(req->authorization,"Basic ",6))
    {
        send_r_bad_request(req);
        return 0;
    }

    base64decode(auth_userpass,req->authorization+6,0x100);

#if !defined(MGMT_LOGIN_LEVEL)
    strcpy(user,nvram_safe_get("user1"));
    strcpy(password,nvram_safe_get("password1"));
        if (user!="") {
            strncpy(admin_passwd,user,strlen(user));
        }
    strcat(admin_passwd,":");
        if (password!="") {
        strcat(admin_passwd,password);
        }
        if ( strcmp(auth_userpass, admin_passwd) !=0 ) {
        send_r_unauthorized(req, auth_realm);
        syslog(6, "[%s login failure] from source %s,", from_subnet(req, NULL), req->remote_ip_addr);
        client_req.authed_send = 1;
        return 0;
        }
#else
	{

		int mgmt_level = 0;
		char *pwd;
		char strbuf[10];

		if ( (pwd = strchr(auth_userpass,':')) == 0 ) {
			//syslog(LOG_ERR, "No user:pass in Basic auth\n");
			//send_r_bad_request(req);
			//return 0;
        		send_r_unauthorized(req, auth_realm);
			syslog(6, "[%s login failure] from source %s,", from_subnet(req, NULL), req->remote_ip_addr);
			client_req.authed_send = 1;
        		return 0;
		}
		*pwd++=0;

		mgmt_level = auth_check_userpass(auth_userpass, pwd, req);
		if ((mgmt_level >= 0) && (client_req.mgmt_level != mgmt_level)) {
			client_req.mgmt_level = mgmt_level;
			sprintf(strbuf, "%d", mgmt_level);
			nvram_set("mgmt_level", strbuf);
		}
		if (mgmt_level < 0) {
        		send_r_unauthorized(req, auth_realm);
			syslog(6, "[%s login failure] from source %s,", from_subnet(req, NULL), req->remote_ip_addr);
			client_req.authed_send = 1;
        		return 0;
		}
	}
#endif
auth_sucess:

    if ((g_bLogin == 0) && client_req.authed_send) {
        syslog(6, "[%s login] from source %s,", from_subnet(req, NULL), req->remote_ip_addr);//yy
        memset(client_req.client_ip, 0x0, MAX_IP_LEN);
        strncpy(client_req.client_ip, req->remote_ip_addr, sizeof(client_req.client_ip));
        memset(client_req.client_macaddress, 0x0, MAX_IP_LEN);
        strncpy(client_req.client_macaddress, getMacAddr(req->remote_ip_addr),
                 MAX_IP_LEN);
        g_bLogin = 1;
        nvram_set("user_login", "1");
        nvram_set("auth_ip",client_req.client_ip);
        nvram_set("auth_mac",client_req.client_macaddress);
        client_req.lastaccess_time=GetSysUpTime();

        return 1;
    }

    return 1;
}
#endif


int process_header_end(request * req)
{
    int need_auth = 0;
    char passwd[40] = "";

#if 0
#if WAN_LAN_CONFLICT
	if (open_hijack_conflict(DNS_HIJACK) != 1)
	{
		if(open_hijack_conflict(WANLAN_CONFLICT)) {
			send_r_wanlan_conflict(req);
            req->status = DONE;
            return 0 ;
		}
	}
#endif
#endif

#if defined(DNI_AUTH)
    if (!auth_authorize(req)) {
        return 0;
    }
#endif

after_auth:
    if (req->method == M_POST) {
        char *tmpfilep = tmpnam(NULL);

        if (!tmpfilep) {
            boa_perror(req, "tmpnam");
            return 0;
        }
        /* open temp file for post data */
        if ((req->post_data_fd = open(tmpfilep, O_RDWR | O_CREAT /* read-write, create */
                                      | O_EXCL, S_IRUSR /* don't overwrite, u+r */
                                      & ~S_IWUSR & ~S_IEXEC /* u-wx */
                                      & ~S_IRWXG /* g-rwx */
                                      & ~S_IRWXO)) == -1) { /* o-rwx */
            boa_perror(req, "tmpfile open");
            return 0;
        }
        req->post_file_name = strdup(tmpfilep);
        free(tmpfilep);
    req->cgi_env[req->cgi_env_index++]  = env_gen("CGI_POST_TMPFILE", req->post_file_name);
        return 1;
    }
    if (req->is_cgi) {
        return init_cgi(req);
    }
    req->status = WRITE;
    return init_get(req);       /* get and head */
}

/*
 * Name: process_option_line
 *
 * Description: Parses the contents of req->header_line and takes
 * appropriate action.
 */

void process_option_line(request * req)
{
    char c, *value, *line = req->header_line;

#ifdef SOAP_ON
    char *p;
    int n;
 #endif

    /* Start by aggressively hacking the in-place copy of the header line */

#ifdef FASCIST_LOGGING
    log_error_time();
    fprintf(stderr, "%s:%d - Parsing \"%s\"\n", __FILE__, __LINE__, line);
#endif

    value = strchr(line, ':');
    if (value == NULL)
        return;
    *value++ = '\0';            /* overwrite the : */
    to_upper(line);             /* header types are case-insensitive */
    while ((c = *value) && (c == ' ' || c == '\t'))
        value++;

    if (!memcmp(line, "IF_MODIFIED_SINCE", 18) && !req->if_modified_since)
        req->if_modified_since = value;

    else if (!memcmp(line, "CONTENT_TYPE", 13) && !req->content_type)
        req->content_type = value;

    else if (!memcmp(line, "CONTENT_LENGTH", 15) && !req->content_length)
        req->content_length = value;

    else if (!memcmp(line, "CONNECTION", 11) &&
             ka_max && req->keepalive != KA_STOPPED) {
        req->keepalive = (!strncasecmp(value, "Keep-Alive", 10) ?
                          KA_ACTIVE : KA_STOPPED);
#if defined(SUPPORT_SHT)
	req->keepalive = KA_STOPPED;
#endif	         
    }
    /* #ifdef ACCEPT_ON */
    else if (!memcmp(line, "ACCEPT", 7))
        add_accept_header(req, value);
    /* #endif */

    else if (!memcmp(line,"AUTHORIZATION",14) && !req->authorization)
            req->authorization = value;

    /* Need agent and referer for logs */
    else if (!memcmp(line, "REFERER", 8))
        req->header_referer = value;
    else if (!memcmp(line, "USER_AGENT", 11))
        req->header_user_agent = value;
	
#ifdef SOAP_ON
    /* SOAPaction */
    else if (!memcmp(line, "SOAPACTION", 11)) {
	p = value;
	n = 0;
	
	while (*p == ':' || *p == ' ' || *p == '\t') p++;
	
	while (p[n] >= ' ') n++;
	
	if ((p[0] == '"' && p[n-1] == '"')
	    || (p[0] == '\'' && p[n-1] == '\'')) {
		p++; n -= 2;
	}

	req->soap_action = p;
	req->soap_actionlen = n;
    }
#endif

    /* Silently ignore unknown header lines unless is_cgi */
    else if (req->is_cgi)
        add_cgi_env(req, line, value);
    return;
}

/*
 * Name: add_accept_header
 * Description: Adds a mime_type to a requests accept char buffer
 *   silently ignore any that don't fit -
 *   shouldn't happen because of relative buffer sizes
 */

void add_accept_header(request * req, char *mime_type)
{
#ifdef ACCEPT_ON
    int l = strlen(req->accept);
    int l2 = strlen(mime_type);

    if ((l + l2 + 2) >= MAX_HEADER_LENGTH)
        return;

    if (req->accept[0] == '\0')
        strcpy(req->accept, mime_type);
    else {
        req->accept[l] = ',';
        req->accept[l + 1] = ' ';
        memcpy(req->accept + l + 2, mime_type, l2 + 1);
        /* the +1 is for the '\0' */
        /*
           sprintf(req->accept + l, ", %s", mime_type);
         */
    }
#endif
}

void free_requests(void)
{
    request *ptr, *next;

    ptr = request_free;
    while (ptr != NULL) {
        next = ptr->next;
        free(ptr);
        ptr = next;
    }
    request_free = NULL;
}
