/* This is a module which is used for source-NAT-P2P.
 * with concept helped by Rusty Russel <rusty at rustcorp.com.au>
 * and with code by Jesse Peng <tzuhsi.peng at msa.hinet.net>
 */
#include <linux/netfilter_ipv4.h>
#include <linux/module.h>
#include <linux/kmod.h>
#include <linux/skbuff.h>
#include <linux/netfilter_ipv4/ip_conntrack.h>
#include <linux/netfilter_ipv4/ip_tables.h>
#include <linux/netfilter_ipv4/ip_nat.h>
#include <linux/netfilter_ipv4/ip_nat_core.h>
#include <linux/netfilter_ipv4/ip_nat_rule.h>
#include <net/ip.h>

#if 1
#define DEBUGP printk
#else
#define DEBUGP(format, args...)
#endif

static int kill_proto(const struct ip_conntrack *i, void *data)
{
	return (i->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.protonum == 
			*((u_int8_t *) data));
}
static unsigned int ipt_snatp2p_target(struct sk_buff **pskb,
		unsigned int hooknum,
		const struct net_device *in,
		const struct net_device *out,		
		const void *targinfo,
		void *userinfo)
{
	struct ip_conntrack *ct;
	enum ip_conntrack_info ctinfo;
	const struct ip_nat_multi_range *mr = targinfo;

	IP_NF_ASSERT(hooknum == NF_IP_POST_ROUTING);
	ct = ip_conntrack_get(*pskb, &ctinfo);
	
	/* Connection must be valid and new. */
	IP_NF_ASSERT(ct && (ctinfo == IP_CT_NEW || ctinfo == IP_CT_RELATED
				|| ctinfo == IP_CT_RELATED + IP_CT_IS_REPLY));
	IP_NF_ASSERT(out);
	ct->status |= IPS_SNATP2P_SRC;

	WRITE_LOCK(&out_ifname_lock);
	strncpy(ct->nat.out_ifname,out->name,IFNAMSIZ);
	//printk("out->name=%s,ct->nat.out_ifname =%s!\n",out->name,ct->nat.out_ifname);
    WRITE_UNLOCK(&out_ifname_lock);

	struct iphdr *iph = ((struct sk_buff *)(*pskb))->nh.iph;
	
	if (iph->ihl > 5) {
		/*struct ip_options *opt;
			
		opt = &(IPCB(*pskb)->opt);	
		if (opt->srr)
			return 1;*/
        //ticket 33: IP option 
        ct->tuplehash[IP_CT_DIR_REPLY].tuple.src.ip =   iph->daddr;
        ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.ip  =   iph->daddr;
	}
	
	return ip_nat_setup_info(ct, targinfo, hooknum);	
}
	
static int ipt_snatp2p_checkentry(const char *tablename,
		const struct ipt_entry *e,
		void *targinfo,
		unsigned int targinfosize,
		unsigned int hook_mask)
{
	struct ip_nat_multi_range *mr = targinfo;
	u_int8_t proto_icmp = IPPROTO_ICMP;
	/* Must be a valid range */
	if (mr->rangesize != 1) {
		printk("SNATP2P: multiple ranges no longer supported\n");
		return 0;
	}

	if (targinfosize != IPT_ALIGN(sizeof(struct ip_nat_multi_range))) {
	   		DEBUGP("SNATP2P: Target size %u wrong for %u ranges\n",
				targinfosize, mr->rangesize);
			return 0;
	}
			
	/* Only allow these for NAT. */
	if (strcmp(tablename, "nat") != 0) {
		DEBUGP("SNATP2P: wrong table %s\n", tablename);
		return 0;
	}

	if (hook_mask & ~(1 << NF_IP_POST_ROUTING)) {
		DEBUGP("SNATP2P: hook mask 0x%x bad\n", hook_mask);
		return 0;
	}
	ip_ct_selective_cleanup(kill_proto, &proto_icmp);

	return 1;
}
								 
static struct ipt_target ipt_snatp2p_reg = {
	.name           = "SNATP2P",
	.target         = ipt_snatp2p_target,
	.checkentry     = ipt_snatp2p_checkentry,
};

static int __init init(void)
{
	if (ipt_register_target(&ipt_snatp2p_reg))
		return -EINVAL;
		
	return 0;
}

static void __exit fini(void)
{		       
	ipt_unregister_target(&ipt_snatp2p_reg);
}
		
module_init(init);
module_exit(fini);
