/*  Copyright (C) 2007 Delta Networks Inc.
  * This module is used by DNI's net-wall for logging packets according to NETGEAR spec.
  * And we are the last match in the netfilter rules to reduce the number of rules.
  */
#include <linux/module.h>
#include <linux/skbuff.h>
#include <linux/spinlock.h>
#include <linux/ip.h>
#include <net/icmp.h>
#include <net/udp.h>
#include <net/tcp.h>
#include <net/route.h>

#include <linux/netfilter_ipv4/ip_tables.h>
#include <linux/netfilter_ipv4/ipt_log.h>

#if 0
#define DEBUGP printk
#else
#define DEBUGP(format, args...)
#endif

static int ipt_log_match(const struct sk_buff *skb,
		const struct net_device *in,
		const struct net_device *out,
		const void *matchinfo,
		int offset,
		const void *hdr,
		u_int16_t datalen,
		int *hotdrop)
{
//	if (net_ratelimit() == 0)
//		return 1;

	
	struct iphdr *iph = skb->nh.iph;
	void *protoh = (u_int32_t *)iph + iph->ihl;
	const struct ipt_logs_info *loginfo = matchinfo;
	char level[4]="<7>";	// LEVEL = LOG_DEBUG
	int remote_access = 0;
	//int dos = 0;
	printk("%s",level);

	if (strstr(loginfo->prefix, "from remote"))
		remote_access = 1;
	/*if (strstr(loginfo->prefix, "DoS Attack"))
		dos = 1;*/
	if (remote_access)	
	{
		do_syslog(6,NULL,0);    //disable console
		printk("%s from %u.%u.%u.%u",loginfo->prefix,NIPQUAD(iph->saddr));
		do_syslog(7,NULL,0);    //enable console
	}

	else
		printk("%s from source %u.%u.%u.%u,",loginfo->prefix,NIPQUAD(iph->saddr));

	switch (iph->protocol) {
        case IPPROTO_TCP: {
                struct tcphdr *tcph = protoh;
		if (remote_access)
		{
                	do_syslog(6,NULL,0);    //disable console
	                printk(":%u to %u.%u.%u.%u:%u,\n",ntohs(tcph->source), NIPQUAD(iph->daddr), ntohs(tcph->dest));
                	do_syslog(7,NULL,0);    //enable console
		}
		else
	                printk(" port %u,\n",ntohs(tcph->source));
                break;
        }
        case IPPROTO_UDP: {
                struct udphdr *udph = protoh;
		if (remote_access)
		{
                	do_syslog(6,NULL,0);    //disable console
	                printk(":%u to %u.%u.%u.%u:%u\n",ntohs(udph->source), NIPQUAD(iph->daddr), ntohs(udph->dest));
                	do_syslog(7,NULL,0);    //enable console
		}
		else
	                printk(" port %u,\n",ntohs(udph->source));
                break;
        }
	case IPPROTO_ICMP: {
                struct icmphdr *icmph = protoh;
                static size_t required_len[NR_ICMP_TYPES+1]
                        = { [ICMP_ECHOREPLY] = 4,
                            [ICMP_DEST_UNREACH]
                            = 8 + sizeof(struct iphdr) + 8,
                            [ICMP_SOURCE_QUENCH]
                            = 8 + sizeof(struct iphdr) + 8,
                            [ICMP_REDIRECT]
                            = 8 + sizeof(struct iphdr) + 8,
                            [ICMP_ECHO] = 4,
                            [ICMP_TIME_EXCEEDED]
                            = 8 + sizeof(struct iphdr) + 8,
                            [ICMP_PARAMETERPROB]
                            = 8 + sizeof(struct iphdr) + 8,
                            [ICMP_TIMESTAMP] = 20,
                            [ICMP_TIMESTAMPREPLY] = 20,
                            [ICMP_ADDRESS] = 12,
                            [ICMP_ADDRESSREPLY] = 12 };

                printk(" type %u\n", icmph->type);
                break;
        }
        /* Max length: 10 "PROTO 255 " */
        default:
                printk("PROTO=%u\n", iph->protocol);
        }
#if 0
	if (has_port && (loginfo->flags & LOG_SPORT))
		printk("[%s] from source: %u.%u.%u.%u, port %u,\n", loginfo->prefix,
				NIPQUAD(iph->saddr), ntohs(tcph->source));
	else if (has_port && (loginfo->flags & LOG_PORT))
		printk("[%s] from %u.%u.%u.%u:%u to %s:%u,\n", loginfo->prefix, NIPQUAD(iph->saddr),
				ntohs(tcph->source), loginfo->destip, ntohs(tcph->dest));
	else
		printk("[%s] from source %u.%u.%u.%u,\n", loginfo->prefix, NIPQUAD(iph->saddr));
#endif	
	return 1;
}

static int ipt_log_check(const char *tablename,
		const struct ipt_ip *ip,
		void *matchinfo,
		unsigned int matchsize,
		unsigned int hook_mask)
{
	struct ipt_logs_info *loginfo = (struct ipt_logs_info *)matchinfo;

	if (matchsize != IPT_ALIGN(sizeof(struct ipt_logs_info))) {
		DEBUGP("log: matchsize %u != %u\n", matchsize, IPT_ALIGN(sizeof(struct ipt_logs_info)));
		return 0;
	}

	if (loginfo->prefix[sizeof(loginfo->prefix)-1] != '\0') {
		DEBUGP("log: prefix term %i\n", loginfo->prefix[sizeof(loginfo->prefix)-1]);
		return 0;
	}

	return 1;
}

static struct ipt_match ipt_log_m = {
	.list 		= {NULL, NULL},
	.name		= "log",
	.match		= ipt_log_match,
	.checkentry	= ipt_log_check,
	.me		= THIS_MODULE,
};

static int __init init(void)
{
	if (ipt_register_match(&ipt_log_m))
		return -EINVAL;

	return 0;
}

static void __exit fini(void)
{
	ipt_unregister_match(&ipt_log_m);
}

module_init(init);
module_exit(fini);
MODULE_LICENSE("GPL");
